-- Look for blocking across the instance
SELECT  tl.resource_type ,
    tl.resource_database_id ,
    tl.resource_associated_entity_id ,
    tl.request_mode ,
    tl.request_session_id ,
    wt.blocking_session_id ,
    wt.wait_type ,
    wt.wait_duration_ms
FROM    sys.dm_tran_locks AS tl
    INNER JOIN sys.dm_os_waiting_tasks AS wt
        ON tl.lock_owner_address = wt.resource_address
ORDER BY wait_duration_ms DESC ;

-- Locks on a specific session_id
SELECT t1.resource_type,
    t1.resource_database_id,
    t1.resource_associated_entity_id,
    t1.request_mode,
    t1.request_session_id,
    t2.blocking_session_id
FROM sys.dm_tran_locks AS t1
    INNER JOIN sys.dm_os_waiting_tasks AS t2
        ON t1.lock_owner_address = t2.resource_address
--  WHERE t1.request_session_id = ISNULL(@spid, t1.requested_session_id)
GO

-- A Blocking Monitor Script written by Grant Fritchey
SELECT  tl.request_session_id AS WaitingSessionID
       ,wt.blocking_session_id AS BlockingSessionID
       ,wt.resource_description
       ,wt.wait_type
       ,wt.wait_duration_ms
       ,DB_NAME(tl.resource_database_id) AS DatabaseName
       ,tl.resource_associated_entity_id AS WaitingAssociatedEntity
       ,tl.resource_type AS WaitingResourceType
       ,tl.request_type AS WaitingRequestType
       ,wrt.[text] AS WaitingTSql
       ,btl.request_type BlockingRequestType
       ,brt.[text] AS BlockingTsql
FROM    sys.dm_tran_locks tl
   JOIN sys.dm_os_waiting_tasks wt
        ON tl.lock_owner_address = wt.resource_address
   JOIN sys.dm_exec_requests wr
        ON wr.session_id = tl.request_session_id
   CROSS APPLY sys.dm_exec_sql_text(wr.sql_handle) AS wrt
   LEFT JOIN sys.dm_exec_requests br
        ON br.session_id = wt.blocking_session_id
   OUTER APPLY sys.dm_exec_sql_text(br.sql_handle) AS brt
   LEFT JOIN sys.dm_tran_locks AS btl
        ON br.session_id = btl.request_session_id;
