/* 
    Execute the below scripts which queries sys.dm_sql_referencing_entities 
    dynamic management function to find out all objects which are referencing 
    to Employee table. 
*/

/* Find all object which are referencing to "Employee" table */
SELECT
  referencing_schema_name + '.' + referencing_entity_name AS ReferencedEntityName
 ,referencing_class_desc AS ReferencingEntityDescription
FROM
  sys.dm_sql_referencing_entities(
    'HumanResources.Employee'
   ,'OBJECT');
GO

/*
    Execute the below scripts which queries sys.dm_sql_referenced_entities 
    dynamic management function to find out all objects which are referenced 
    to usp_GetEmployeeName stored procedure.
*/

/* Find all object which are referenced by "dbo.ufnGetProductListPrice" stored procedure */
SELECT 
  referenced_schema_name +'.'+ referenced_entity_name AS ReferencedEntityName, 
  referenced_minor_name AS ReferencedMinorName
FROM sys.dm_sql_referenced_entities ('dbo.ufnGetProductListPrice', 'OBJECT');
GO

/* 
    Execute the below scripts which queries sys.sql_expression_dependencies 
    system view to find out all the objects which are referenced by 
    dbo.ufnGetProductListPrice stored procedure.
*/

 /* Identifying Object Dependencies */
SELECT SCHEMA_NAME(O.SCHEMA_ID) +'.'+ o.name AS ReferencingObject, 
   SED.referenced_schema_name +'.'+SED.referenced_entity_name AS ReferencedObject
FROM sys.all_objects O 
   INNER JOIN sys.sql_expression_dependencies SED ON O.OBJECT_ID=SED.REFERENCING_ID 
WHERE O.name = 'ufnGetProductListPrice'
GO

