-- See my blog for more information:
-- http://sqlblog.com/blogs/kalen_delaney/archive/2006/10/13/alter-table-will-not-reclaim-space.aspx

-- Create a table with two large char columns and a smallint. 
-- The initial length of each row will be just a bit more than 3000 bytes, 
-- out of a maximum row length of 8060 bytes. 

use pubs
go

IF EXISTS (SELECT 1 FROM sys.tables WHERE name = 'bigchange')
   DROP TABLE bigchange;
GO

CREATE TABLE bigchange
(col1 smallint, col2 char(2000), col3 char(1000));


-- Now alter the second column to increase its length to 3000 bytes. 
-- The length of a row should now be just a bit more than 4000 bytes.

ALTER TABLE bigchange 
   ALTER COLUMN col2 char(3000);

-- Now try and add another 3000 byte column, which should bring the 
-- length to just over 7000 bytes. This will fail:

ALTER TABLE bigchange 
      ADD col4 char(3000);


-- However, if you just create a table with two 3000-byte columns 
-- and a 1000-byte column, there will be no problem.

IF EXISTS (SELECT 1 FROM sys.tables WHERE name = 'nochange')
   DROP TABLE nochange;
GO

CREATE TABLE nochange
(col1 smallint, 
 col2 char(3000), 
 col3 char(1000), 
 col4 char(3000));
GO

 

-- Now run the query to see the metadata for where each column will start. 

SELECT  c.name AS column_name, column_id, max_inrow_length,
      pc.system_type_id, leaf_offset 
 FROM sys.system_internals_partition_columns pc
    JOIN sys.partitions p 
        ON p.partition_id = pc.partition_id
    JOIN sys.columns c
         ON column_id = partition_column_id 
            AND c.object_id = p.object_id
WHERE p.object_id=object_id('bigchange');
  


-- SQL Server does not reuse the original bytes in the row when a rows length is altered,
-- You may end up not being able to use the full allowable row length
--  if youve altered column lengths. 