--  Hugo Kornelius, MVP Deep Dives 2008
-- 
-- In current versions of SQL Server, the only way to calculate running totals in declarative
-- code is to join each row from the table to all preceding rows for the same customer
-- from itself, adding all those joined rows together to calculate the running total.
--
SELECT s.CustomerID, s.OrderDate, s.SalesOrderID, s.TotalDue,
   SUM(s2.TotalDue) AS RunningTotal
FROM Sales.SalesOrderHeader AS s
   INNER JOIN Sales.SalesOrderHeader AS s2
      ON s2.CustomerID = s.CustomerID
   AND ( s2.OrderDate < s.OrderDate
     OR ( s2.OrderDate = s.OrderDate
     AND s2.SalesOrderID <= s.SalesOrderID))
GROUP BY s.CustomerID, s.OrderDate, s.SalesOrderID, s.TotalDue
ORDER BY s.CustomerID, s.OrderDate, s.SalesOrderID;
GO

DECLARE @Results TABLE
  (CustomerID int NOT NULL,
  OrderDate datetime NOT NULL,
  SalesOrderID int NOT NULL,
  TotalDue money NOT NULL,
  RunningTotal money NULL,
  PRIMARY KEY (CustomerID, OrderDate, SalesOrderID));
INSERT INTO @Results(CustomerID, OrderDate, SalesOrderID, TotalDue)
  SELECT CustomerID, OrderDate, SalesOrderID, TotalDue
  FROM Sales.SalesOrderHeader;
DECLARE @CustomerID int, @OrderDate datetime,
  @SalesOrderID int, @TotalDue money,
  @CurrCustomerID int, @RunningTotal money;
SET @CurrCustomerID = 0;
SET @RunningTotal = 0;
DECLARE SalesCursor CURSOR STATIC READ_ONLY
  FOR SELECT CustomerID, OrderDate, SalesOrderID, TotalDue
  FROM @Results
  ORDER BY CustomerID, OrderDate, SalesOrderID; 
OPEN SalesCursor;
FETCH NEXT FROM SalesCursor
  INTO @CustomerID, @OrderDate, @SalesOrderID, @TotalDue;
WHILE @@FETCH_STATUS = 0
BEGIN;
	IF @CustomerID <> @CurrCustomerID
		BEGIN;
		SET @CurrCustomerID = @CustomerID;
		SET @RunningTotal = 0;
		END;
	SET @RunningTotal = @RunningTotal + @TotalDue;
	UPDATE @Results
	SET RunningTotal = @RunningTotal
	WHERE CustomerID = @CustomerID
	  AND OrderDate = @OrderDate
	  AND SalesOrderID = @SalesOrderID;
	FETCH NEXT FROM SalesCursor
	  INTO @CustomerID, @OrderDate, @SalesOrderID, @TotalDue;
END;
CLOSE SalesCursor;
DEALLOCATE SalesCursor;
SELECT CustomerID, OrderDate, SalesOrderID, TotalDue, RunningTotal
  FROM @Results
  ORDER BY CustomerID, OrderDate, SalesOrderID;
GO

DECLARE @Results TABLE
	(CustomerID int NOT NULL,
	OrderDate datetime NOT NULL,
	SalesOrderID int NOT NULL,
	TotalDue money NOT NULL,
	RunningTotal money NULL,
	Rnk int NOT NULL,
	PRIMARY KEY (Rnk, CustomerID));
INSERT INTO @Results
	(CustomerID, OrderDate, SalesOrderID,
	TotalDue, RunningTotal, Rnk)
SELECT CustomerID, OrderDate, SalesOrderID, TotalDue, TotalDue,
	RANK() OVER (PARTITION BY CustomerID
	ORDER BY OrderDate, SalesOrderID)
	FROM Sales.SalesOrderHeader;
DECLARE @Rank int, @RowCount int;
SET @Rank = 1;
SET @RowCount = 1;
WHILE @RowCount > 0
BEGIN;
	SET @Rank = @Rank + 1;
	UPDATE nxt
	SET RunningTotal = prv.RunningTotal + nxt.TotalDue
	FROM @Results AS nxt
		INNER JOIN @Results AS prv
		ON prv.CustomerID = nxt.CustomerID
		AND prv.Rnk = @Rank- 1
	WHERE nxt.Rnk = @Rank;
	SET @RowCount = @@ROWCOUNT;
END;
SELECT CustomerID, OrderDate, SalesOrderID, TotalDue, RunningTotal
	FROM @Results
	ORDER BY CustomerID, OrderDate, SalesOrderID;
GO

