/*
Execution Related DMV
Server Scope
Returns aggregate performance statistics for cached query plans. The view contains one row per query statement within the cached plan,
and the lifetime of the rows are tied to the plan itself. When a plan is removed from the cache, the corresponding rows are eliminated
from this view. 
This particular utilization shows the top 100 stored procedures ranked by number of calls
*/

DECLARE @StartDate DATETIME ;
DECLARE @EndDate DATETIME ;
DECLARE @CurrentTime DATETIME ; 

SET @StartDate = '2008-02-04' ;
SET @EndDate = GETDATE() ;
SET @CurrentTime = GETDATE() ;

-- Get Top 100 executed SP's ordered by calls/sec
SELECT TOP 100 SCHEMA_NAME([schema_id]) AS schemaName
   ,p.[name] AS objectName
   ,qs.execution_count AS executionCount
   ,qs.total_worker_time / qs.execution_count AS avgWorkerTime
   ,qs.total_worker_time AS totalWorkerTime
   ,qs.total_elapsed_time / qs.execution_count AS avgElapsedTime
   ,qs.max_logical_reads
   ,qs.max_logical_writes
   ,qs.total_physical_reads
   ,CONVERT(DECIMAL(20, 4), qs.total_physical_reads) / CONVERT(DECIMAL(20,4), qs.execution_count) AS avgPhysicalReads
   ,qs.creation_time
   ,DATEDIFF(MI,qs.creation_time,@CurrentTime) AS ageInCache
   ,qs.execution_count / DATEDIFF(SS,qs.creation_time,@CurrentTime) AS callsPerSecond
   ,qs.last_execution_time
   ,SUBSTRING(qt.text,qs.statement_start_offset / 2,( CASE WHEN qs.statement_end_offset = -1 THEN LEN(CONVERT(NVARCHAR(MAX),qt.text)) * 2
                                                           ELSE qs.statement_end_offset
                                                      END - qs.statement_start_offset ) / 2) AS individualQueryText
FROM sys.dm_exec_query_stats AS qs
    CROSS APPLY sys.dm_exec_sql_text(qs.sql_handle) AS qt
    INNER JOIN sys.procedures AS p ON qt.objectid = p.[object_id]
WHERE qt.dbid = DB_ID()
    AND (
          qs.creation_time BETWEEN @StartDate AND @EndDate
          OR qs.last_execution_time BETWEEN @StartDate AND @EndDate
        )
ORDER BY callsPerSecond DESC ;