-- ////////////////////////////////////////////////////////////////////////////
-- //$Revision: 1 $
-- //$Date: 2010-Apr-28 8:49a $
-- //$Modtime: 2010-Apr-28 8:44a $
-- //$Workfile: TableStats2.sql $
-- ////////////////////////////////////////////////////////////////////////////
-- ////////////////////////////////////////////////////////////////////////////
-- //
-- //  TableStats2
-- //
-- //  Modification history:
-- //  Rev#  Date       Author     Description
-- //   ----- --------  ---------  --------------------------------------------
-- //  1.0   2010-Apr-28 R Gosling  Checked into SS
-- ////////////////////////////////////////////////////////////////////////////

---==========================================================
-- Get the details on indexes, filegroups and partitions
-- for all tables in a database
--============================================================
SELECT  OBJECT_NAME(i.id) AS objectName
       ,i.[name] AS IndexName
       ,f.name AS FileGroupName
       ,SUM(a.total_pages) AS totalPages
       ,SUM(a.used_pages) AS usedPages
       ,SUM(a.data_pages) AS dataPages
       ,( SUM(a.total_pages) * 8 ) / 1024 AS TotalMB
       ,( SUM(a.used_pages) * 8 ) / 1024 AS UsedMB
       ,( SUM(a.data_pages) * 8 ) / 1024 AS DataMB
       ,max(i.Rows) as rCount
FROM    sys.sysindexes i
        INNER JOIN SYS.INDEXES x ON x.index_id=i.indid
        INNER JOIN sys.filegroups f ON f.data_space_id=x.data_space_id
        INNER JOIN sys.partitions p ON i.id=p.object_id
                                       AND i.indid=p.index_id
        INNER JOIN sys.allocation_units a ON p.partition_id=a.container_id
GROUP BY i.id
       ,i.indid
       ,i.[name]
       ,f.name
ORDER BY SUM(a.total_pages) DESC
       ,OBJECT_NAME(i.id)
 

-- Server wide look at log space
DBCC SQLPERF( LOGSPACE )
       
-- Get the current file layout
exec sp_helpfile

-- Get the file group and available space in each
SELECT  name
       ,size / 128.0 - CAST(FILEPROPERTY(name, 'SpaceUsed') AS INT) / 128.0 AS 'Avl Space (MB)'
FROM    sys.database_files ;