-- TRANCOUNT examples
Use tempdb
GO
IF EXISTS (SELECT name 
	   FROM   sysobjects 
	   WHERE  name = N'TranTest_Insert' 
	   AND 	  type = 'P')
    DROP PROCEDURE TranTest_Insert
GO
IF EXISTS(SELECT name 
	  FROM 	 sysobjects 
	  WHERE  name = N'TranTest' 
	  AND 	 type = 'U')
    DROP TABLE TranTest
GO

CREATE TABLE TranTest ( RowNumber INT PRIMARY KEY)
GO



CREATE PROCEDURE TranTest_Insert 
	@RowNumber INT = NULL 
AS

Print 'TranTest_Insert Starting @@TRANCOUNT: ' + CAST(@@TRANCOUNT AS VARCHAR)

BEGIN TRAN

Print 'TranTest_Insert @@TRANCOUNT before work: ' + CAST(@@TRANCOUNT AS VARCHAR)

IF @RowNumber IS NOT NULL
  BEGIN

	BEGIN TRY
		INSERT TranTest (RowNumber)
		VALUES ( @RowNumber )

		COMMIT TRAN
		Print 'TranTest_Insert ending @@TRANCOUNT: ' + CAST(@@TRANCOUNT AS VARCHAR)
	END TRY

	BEGIN CATCH
		ROLLBACK TRAN
		PRINT 'The stored procedure is rolling back...'
		Print 'TranTest_Insert ending @@TRANCOUNT: ' + CAST(@@TRANCOUNT AS VARCHAR)
		return 1		
	END CATCH

  END
GO


-- Basic COMMIT and ROLLBACK -------------------------------------------

Use tempdb
GO
-- Results to Text
WHILE @@trancount > 0 rollback tran
GO
IF EXISTS(SELECT name 
	  FROM 	 sysobjects 
	  WHERE  name = N'Tran1' 
	  AND 	 type = 'U')
    DROP TABLE Tran1
GO

CREATE TABLE Tran1 ( V1 INT NULL )
GO
SET NOCOUNT OFF
GO



-- Example with COMMIT -------------------------------------------------
Print 'Starting TranCount=' + CAST(@@TRANCOUNT AS VARCHAR)
BEGIN TRAN
    INSERT Tran1 VALUES (1)
  
    BEGIN TRAN
        INSERT Tran1 VALUES (2)
        Print 'InnerMostTranCount=' + CAST(@@TRANCOUNT AS VARCHAR)
    COMMIT TRAN
  
    Print 'MiddleTranCount=' + CAST(@@TRANCOUNT AS VARCHAR)

COMMIT TRAN

Print 'OuterTranCount=' + CAST(@@TRANCOUNT AS VARCHAR)

SELECT * FROM Tran1
DELETE Tran1
GO



-- Example with ROLLBACK ------------------------------------------------
BEGIN TRAN
	INSERT Tran1 VALUES (1)
   
	BEGIN TRAN
		INSERT Tran1 VALUES (2)
		Print 'InnerMostTranCount=' + CAST(@@TRANCOUNT AS VARCHAR)
	ROLLBACK TRAN

	Print 'MiddleTranCount=' + CAST(@@TRANCOUNT AS VARCHAR)

ROLLBACK TRAN

Print 'OuterTranCount=' + CAST(@@TRANCOUNT AS VARCHAR)
SELECT * FROM Tran1

DELETE Tran1
GO


-- Example with COMMIT and ROLLBACK ------------------------------------
BEGIN TRAN
	INSERT Tran1 VALUES (1)
	
	BEGIN TRAN
		INSERT Tran1 VALUES (2)
		Print 'InnerMostTranCount=' + CAST(@@TRANCOUNT AS VARCHAR)
	COMMIT TRAN
	
	Print 'MiddleTranCount=' + CAST(@@TRANCOUNT AS VARCHAR)

ROLLBACK TRAN
Print 'OuterTranCount=' + CAST(@@TRANCOUNT AS VARCHAR)
SELECT * FROM Tran1

DELETE Tran1

while @@trancount > 0 rollback tran


-- Combined TRANCOUNT with COMMIT and ROLLBACK

Use tempdb
GO
-- Results to Text
WHILE @@TRANCOUNT > 0 ROLLBACK TRAN

DELETE TranTest
GO
Declare @return INT
BEGIN TRAN

	EXEC @return = TranTest_Insert 1
	PRINT 'First return: ' + CAST(@return as VARCHAR)
	if @return <> 0
		BEGIN
			ROLLBACK TRAN
			goto done
		END
	Print 'TranCount: ' + CAST(@@TRANCOUNT AS VARCHAR)

	PRINT ''
	PRINT '------------------------------------------------------------------------'
	PRINT ''


	EXEC @return = TranTest_Insert 1  -- Issues a ROLLBACK which causes an error
	PRINT 'Second return: ' + CAST(@return as VARCHAR)
	if @return <> 0
		BEGIN
			ROLLBACK TRAN -- Should check @@TRANCOUNT
			goto done
		END

COMMIT TRAN

done:

Print 'Final TranCount: ' + CAST(@@TRANCOUNT AS VARCHAR)

SELECT * FROM TranTest


/*

WHILE @@TRANCOUNT > 0
     ROLLBACK TRAN

*/