-- Create the demo table.
-- Note that the PRIMARY KEY implicitly creates a clustered index.
CREATE TABLE dbo.Persons
  (PersonID  int           NOT NULL IDENTITY,
   FirstName varchar(20)   NOT NULL,
   LastName  varchar(30)   NOT NULL,
   Email     varchar(200)  NOT NULL,
   EmailLen  int           NOT NULL,
   Other     varchar(1000) NOT NULL,           -- Mimic large table size.
			 CONSTRAINT PK_Persons PRIMARY KEY(PersonID));
go

-- Add an index that won't be used in the demo.
-- (Required for demo of index usage stats)
CREATE NONCLUSTERED INDEX ix_Persons_Email
ON     dbo.Persons(Email)
WHERE  EmailLen > 1;

-- Test data is based on (read: shamelessly copied from) AdventureWorks,
-- so you must have that demo database installed when running this script.
-- AdventureWorks can be downloaded from http://msftdbprodsamples.codeplex.com/.

-- Enable rowcount feedback (to see the exact row numbers)
SET NOCOUNT OFF;

-- Fill the table with all (approximately 20,000) persons from AdventureWorks
PRINT CAST(SYSDATETIME() AS varchar(23)) + ': start populating table.'
INSERT INTO dbo.Persons
           (FirstName,             LastName,
            Email,                 EmailLen,
            Other)
SELECT      LEFT(FirstName, 20),   LEFT(LastName, 30),
            EmailPromotion,          LEN(EmailPromotion),
            RTRIM(REPLICATE(EmailPromotion + ' ', 10))
FROM        AdventureWorks2012.Person.Person;

-- Double up a few times, to bump the test data to over a million rows
-- 20,000 to 40,000
INSERT INTO dbo.Persons
           (FirstName,             LastName,
            Email,                 EmailLen,              Other)
SELECT      FirstName,             LastName,
            Email,                 EmailLen,              Other
FROM        dbo.Persons;
-- 40,000 to 80,000
INSERT INTO dbo.Persons
           (FirstName,             LastName,
            Email,                 EmailLen,              Other)
SELECT      FirstName,             LastName,
            Email,                 EmailLen,              Other
FROM        dbo.Persons;
-- 80,000 to 160,000
INSERT INTO dbo.Persons
           (FirstName,             LastName,
            Email,                 EmailLen,              Other)
SELECT      FirstName,             LastName,
            Email,                 EmailLen,              Other
FROM        dbo.Persons;
-- 160,000 to 320,000
INSERT INTO dbo.Persons
           (FirstName,             LastName,
            Email,                 EmailLen,              Other)
SELECT      FirstName,             LastName,
            Email,                 EmailLen,              Other
FROM        dbo.Persons;
-- 320,000 to 640,000
INSERT INTO dbo.Persons
           (FirstName,             LastName,
            Email,                 EmailLen,              Other)
SELECT      FirstName,             LastName,
            Email,                 EmailLen,              Other
FROM        dbo.Persons;
-- 640,000 to 1,280,000
INSERT INTO dbo.Persons
           (FirstName,             LastName,
            Email,                 EmailLen,              Other)
SELECT      FirstName,             LastName,
            Email,                 EmailLen,              Other
FROM        dbo.Persons;

-- Add a nonclustered index to both tables.
PRINT CAST(SYSDATETIME() AS varchar(23)) + ': start creating nonclustered index.'
CREATE NONCLUSTERED INDEX ix_PersonName
	ON dbo.Persons (LastName, FirstName);
